/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2000 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <math.h>
#include <stdio.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "main.h"
#include "sg_text_dialog.h"

static void sg_menu_append_rows				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_append_cols				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_insert_rows				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_insert_cols				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_remove_rows				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_remove_cols				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_begin				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_end					(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_reset				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_cut					(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_copy				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_paste				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_clear				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_clear_all				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_col_properties			(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_set_values				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_mask				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_export				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_import				(GtkWidget *widget, 
							 gpointer data);
static void sg_menu_ascii				(GtkWidget *widget, 
							 gpointer data);


void
sg_worksheet_menu_show(GtkWidget *widget, GdkEventButton *event)
{
  GList *w;
  SGworksheet *worksheet = NULL;
  GtkWidget *data;
  GdkModifierType mods;

  w = worksheets;
  while(w){
    worksheet = (SGworksheet *)w->data;
    data = worksheet->sheet;
    if(data && data == widget) break;
    w = w->next;
  }

  active_worksheet = worksheet;

  gdk_window_get_pointer(widget->window, NULL, NULL, &mods);
  if(!(mods & GDK_BUTTON3_MASK)) return;

  gtk_menu_popup(GTK_MENU(worksheet_menu), NULL, NULL, NULL, NULL,
                 event->button, event->time);
}

void
sg_worksheet_build_menu()
{
  GtkWidget *item;
  GtkWidget *sub_menu;


  /* FILE MENU */

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("Export");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_export, NULL);

  item=gtk_menu_item_new_with_label("Import");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_import, NULL);

  item=gtk_menu_item_new_with_label("ASCII Options");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_ascii, NULL);


  worksheet_menu = gtk_menu_new();
  item=gtk_menu_item_new_with_label("File");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(worksheet_menu),item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);


  /* EDIT MENU */

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("Cut");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_cut, NULL);

  item=gtk_menu_item_new_with_label("Copy");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_copy, NULL);

  item=gtk_menu_item_new_with_label("Paste");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_paste, NULL);

  item=gtk_menu_item_new_with_label("Clear");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_clear, NULL);

  item = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_widget_show(item);

  item=gtk_menu_item_new_with_label("Set as Begin");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_begin, NULL);

  item=gtk_menu_item_new_with_label("Set as End");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_end, NULL);

  item=gtk_menu_item_new_with_label("Reset to full range");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_reset, NULL);

  item=gtk_menu_item_new_with_label("Clear worksheet");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_clear_all, NULL);

  item=gtk_menu_item_new_with_label("Edit");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(worksheet_menu),item);


  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);


  /* COLUMN MENU */

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("Add new columns");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_append_cols, NULL);

  item=gtk_menu_item_new_with_label("Insert columns");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_insert_cols, NULL);

  item=gtk_menu_item_new_with_label("Remove columns");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_remove_cols, NULL);

  item=gtk_menu_item_new_with_label("Properties");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_col_properties, NULL);

  item=gtk_menu_item_new_with_label("Column");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(worksheet_menu),item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);


  /* ROW MENU */

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("Add new rows");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_append_rows, NULL);

  item=gtk_menu_item_new_with_label("Insert rows");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_insert_rows, NULL);

  item=gtk_menu_item_new_with_label("Remove rows");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_remove_rows, NULL);

  item=gtk_menu_item_new_with_label("Row");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(worksheet_menu),item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);


  /* ANALYSIS MENU */

  sub_menu = gtk_menu_new();

  item=gtk_menu_item_new_with_label("Set column values");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_set_values, NULL);

  item=gtk_menu_item_new_with_label("Mask data");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(sub_menu),item);
  gtk_signal_connect(GTK_OBJECT(item),"activate",sg_menu_mask, NULL);

  item=gtk_menu_item_new_with_label("Analysis");
  gtk_widget_show(item);
  gtk_menu_append(GTK_MENU(worksheet_menu),item);

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub_menu);

}

static void
sg_menu_append_rows(GtkWidget *widget, gpointer data)
{
  sg_append_dialog(active_worksheet, GTK_ORIENTATION_VERTICAL);
}

static void
sg_menu_append_cols(GtkWidget *widget, gpointer data)
{
  sg_append_dialog(active_worksheet, GTK_ORIENTATION_HORIZONTAL);
}

static void
sg_menu_insert_rows(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;
  GtkSheetRange range;
  gint nrows;
  
  sheet = GTK_SHEET(active_worksheet->sheet);
  range = sheet->range;

  nrows = range.rowi - range.row0 + 1;

  if(sheet->state != GTK_SHEET_ROW_SELECTED) return;

  sg_worksheet_insert_rows(active_worksheet, sheet->range.row0,
                           sheet->range.rowi - sheet->range.row0 + 1);

}

static void
sg_menu_insert_cols(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);

  if(sheet->state == GTK_SHEET_COLUMN_SELECTED){
    sg_worksheet_insert_columns(active_worksheet, sheet->range.col0,
                                sheet->range.coli - sheet->range.col0 + 1);
  }

}

static void
sg_menu_remove_rows(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;
  GtkSheetRange range;
  gint nrows;
  
  sheet = GTK_SHEET(active_worksheet->sheet);

  if(sheet->maxrow < 2) return;

  range = sheet->range;
  nrows = range.rowi - range.row0 + 1;

  if(sheet->state != GTK_SHEET_ROW_SELECTED) return;

  sg_worksheet_delete_rows(active_worksheet, sheet->range.row0, nrows);

}

static void
sg_menu_remove_cols(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;
  GtkSheetRange range;
  gint ncols;
  
  sheet = GTK_SHEET(active_worksheet->sheet);

  if(sheet->maxcol < 2) return;

  range = sheet->range;
  ncols = range.coli - range.col0 + 1;

  if(sheet->state == GTK_SHEET_COLUMN_SELECTED)
    sg_worksheet_delete_columns(active_worksheet, sheet->range.col0, ncols);
}

static void
sg_menu_begin(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);
  sg_worksheet_set_begin(active_worksheet, sheet->range.row0);
}

static void
sg_menu_end(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);
  sg_worksheet_set_end(active_worksheet, sheet->range.row0);
}

static void
sg_menu_reset(GtkWidget *widget, gpointer data)
{
  sg_worksheet_reset(active_worksheet);
}

static void
sg_menu_cut(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);
  sg_clipboard_cut(sheet);
}

static void
sg_menu_copy(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);
  sg_clipboard_copy(sheet);
}

static void
sg_menu_paste(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);
  sg_clipboard_paste(sheet);
}

static void
sg_menu_clear(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);
  gtk_sheet_range_clear(sheet, &sheet->range);
}

static void
sg_menu_clear_all(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);
  gtk_sheet_range_clear(sheet, NULL);
}

static void
sg_menu_col_properties(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);

  if(sheet->state == GTK_SHEET_COLUMN_SELECTED)
     sg_column_dialog(active_worksheet, sheet->range.col0);
}

static void
sg_menu_set_values(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);

  if(sheet->state == GTK_SHEET_COLUMN_SELECTED)
     sg_formula_dialog(active_worksheet, sheet->range.col0);

}
static void
sg_menu_mask(GtkWidget *widget, gpointer data)
{
  GtkSheet *sheet;

  sheet = GTK_SHEET(active_worksheet->sheet);

  if(sheet->state == GTK_SHEET_COLUMN_SELECTED)
     sg_formula_dialog(active_worksheet, sheet->range.col0);

}

static void
sg_menu_export(GtkWidget *widget, gpointer data)
{
  sg_worksheet_export();
}

static void
sg_menu_import(GtkWidget *widget, gpointer data)
{
  sg_worksheet_import();
}

static void
sg_menu_ascii(GtkWidget *widget, gpointer data)
{
  sg_import_dialog(NULL,NULL);
}

